<?php

namespace PME\Controllers;

use Symfony\Component\Config\FileLocator;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use PME\Util\Messages;

class LoginController {
    
    private $connection;
    private $environment;
    private $productId;
    private $loginUrl;
    private $productPath;
    protected $errorCode = [
		"ORA-01017" => "ERROR_SENHAUSUARIONEGADO"
	];
    
    public function __construct($connection, $environment, $productId, $loginUrl, $productPath) {
        $this->connection = $connection;
        $this->environment = $environment;
        $this->productId = $productId;
        $this->loginUrl = $loginUrl;
        $this->productPath = $productPath;
    }

    private function buildData($conditions) {
        $email = $conditions['user'];
        $password = $conditions['password'];
        $requester_url = null;
        $service = 'false';
        $host = 'false';
        $port = 'false';
        return '{"filter":[{"name":"EMAIL","operator":"=","value":"'.$email.'"},{"name":"PASSWORD","operator":"=","value":"'.$password.'"},{"name":"REQUESTER_URL","operator":"=","value":"'.$requester_url.'"},{"name":"PRODUCT_ID","operator":"=","value":"'.$this->productId.'"},{"name":"HOST","operator":"=","value":'.$host.'},{"name":"PORT","operator":"=","value":'.$port.'},{"name":"SERVICE","operator":"=","value":"'.$service.'"}],"page":1,"requestType":"FilterData"}';
    }

    private function buildHeader() {
        $header = array();
        $header[] = 'Content-Type: application/json';
        $header[] = 'Accept: application/json, text/plain, */*';
        $header[] = 'Connection: keep-alive';
        return $header;
    }
    
    private function getUrlLogin() {
        return $_SERVER['HTTP_HOST'].$this->productPath.$this->loginUrl;
    }
    
    private function getParameters() {
        $instanceManager = \Zeedhi\Framework\DependencyInjection\InstanceManager::getInstance();
        $container = $instanceManager->getContainer();
		
		if (!$container->hasParameter('projectid')) {
		    throw new \Exception("Has no projectId parameter.");
		}
		
		return $container->getParameter('projectid');
		
    }
    
    /*
    public function authenticate(Request $request, Response $response) {
        
        $conditions = $request->getParameters();
        try {    
            $data = $this->buildData($conditions);
            $header = $this->buildHeader();
            $result = $this->send('/login', $header, $data);
            
            // preg_match_all('/^Set-Cookie:\s*PHPSESSID=([^;]*)/mi', $result, $cookie);
            preg_match_all('/^({"error":.*)$/mi', $result, $dataset);

            if ($dataset[0]) {
                $dataset = (array) (json_decode($dataset[1][0]));
                $json = (array) json_decode($dataset['error']);

                $error = $dataset['error'];
                if(!empty($json['CODE'])) $error = $json['CODE']; 

                throw new \Exception($error);
            } else {
                preg_match_all('/^({"dataset":.*)$/mi', $result, $dataset);
                $dataset = (array) (json_decode($dataset[1][0]));
                
                if (!isset($dataset[0])) {
                    $dataSet = (array) ($dataset['dataset']->userData);
                    
                    $loginUser  = $dataSet['USER'];
                    $loginToken = $dataSet['TOKEN'];
                    $isLogged   = $dataSet['LOGGED'];
                    
                    $this->requestListenerOAuth->checkAccessAndSetSession($loginToken);
                      
                    $sessionData = $this->startSession(
                        $loginUser,
                        $loginToken
                    );
                    $info = $this->setEnvironmentInfo($dataSet, $conditions);
                    $dataSet['CDFILIALSELECIONADA'] = $info['CDFILIALSELECIONADA'];
                    $dataSet['NRORGSELECIONADA'] = $info['NRORG'];
                    $dataSet['LANGUAGE'] = $this->environment->getLanguage();
                    
                    if (is_null($sessionData)) {
                        throw new \Exception(Messages::SESS_FAILED_TO_START);
                    }
                    
                    $loginData = array_merge($dataSet, $sessionData);
                    
                    $response->addDataSet(new DataSet('system', $loginData));
                } else {
                    throw new \Exception('Error while trying to login.');
                }
            }
        } catch(\Exception $e) {
            $errorMessage = $e->getMessage();
			foreach($this->errorCode as $key => $val){
				if(strpos($errorMessage, $key)) $errorMessage = $val;
			}
            $response->setError(new Error($errorMessage, 1));
        }
    }
    
    public function startSession (
        string  $loginUser,
        string  $loginToken,
        bool    $isLogged = true
    ) {
        try {
            $modulesVersionError = null;
            $loginUserInfo = @json_decode(LibUtilities::zeedhiDecrypt($loginUser), true);
            
            if(empty($loginUserInfo)) {
                $loginUserInfo = InstanceProvider::getSession()->get('LOGIN_USER_INFO');
            } else {
                unset($loginUserInfo['CURRENT_SECONDS']);
            }
            if(isset($loginUserInfo['USER_DATA']) && !empty($loginUserInfo['USER_DATA'])) {
                $userData = $loginUserInfo['USER_DATA'];
            } else {
                $userData = $this->apiService->getUserData($loginUserInfo['USER_EMAIL'], null);
            }
            
            $userData = $this->loginService->restoreSession($userData, $loginUserInfo, $isLogged, $modulesVersionError, $loginToken);
            
            return $userData;
        } catch(\Exception $e) {
            return null;
        }
    }
    
    */
    public function requestNewPassword(Request $request, Response $response) {
        try {   
            $conditions = $request->getParameters();
            $email = $conditions['email'];
            $header = $this->buildHeader();
            $data = '{"filter":[{ "name":"EMAIL", "operator":"=", "value":"'.$email.'" }],"page":1,"requestType":"FilterData"}';
            $result = $this->send('/requestNewPassword', $header, $data);
            
            // preg_match_all('/^Set-Cookie:\s*PHPSESSID=([^;]*)/mi', $result, $cookie);
            preg_match_all('/^({"error":.*)$/mi', $result, $dataset);
            if ($dataset[0]) {
                $dataset = (array) (json_decode($dataset[1][0]));
                throw new \Exception($dataset['error']);
            }else $response->addDataSet(new DataSet('response', [$result]));    
            
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function updatePassword( Request $request, Response $response) {
        try {    
            $conditions = $request->getParameters();
            $header = $this->buildHeader();
            $data = '{"filter":[{ "name":"EMAIL", "operator":"=", "value":"'.$conditions['email'].'" }, { "name":"ACTUAL_PASSWORD", "operator":"=", "value":"'.$conditions['actualPassword'].'" }, { "name":"NEW_PASSWORD", "operator":"=", "value":"'.$conditions['newPassword'].'" }, { "name":"CONFIRM_NEW_PASSWORD", "operator":"=", "value":"'.$conditions['confirmPassword'].'" }],"page":1,"requestType":"FilterData"}';
            $result = $this->send('/updatePassword', $header, $data);
       
            // preg_match_all('/^Set-Cookie:\s*PHPSESSID=([^;]*)/mi', $result, $cookie);
            preg_match_all('/^({"error":.*)$/mi', $result, $dataset);
            
            if ($dataset[0]) {
                $dataset = (array) (json_decode($dataset[1][0]));
                throw new \Exception($dataset['error']);
            }else $response->addDataSet(new DataSet('response', [$result]));    
            
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    private function send($route, $header, $data){
        $loginUrl = $this->getUrlLogin();
        $session = curl_init($loginUrl.$route);
        
        curl_setopt($session, CURLOPT_HTTPHEADER, $header);
        curl_setopt($session, CURLOPT_POST, true);
        curl_setopt($session, CURLOPT_POSTFIELDS, $data);
        curl_setopt($session, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($session, CURLOPT_HEADER, 1);
        $result = curl_exec($session);
        curl_close($session);
        return $result;
    }
    
    private function setEnvironmentInfo ($userData, $conditions){
        $info = $this->getUserInfo($userData['USER_ID']);
        $this->environment->setCdOperador($info['CDOPERADOR']);
        $this->environment->setNmOperador($info['NMOPERADOR']);
        $this->environment->setNrOrgTrab($info['NRORGTRAB']);
        $this->environment->setNrOrg($info['NRORG']);
        $this->environment->setCdFilial($info['CDFILIALSELECIONADA']);
        $this->environment->setLanguage($conditions['language']);
        return $info;
    }
    
    public function userInfo(Request\Row $request, Response $response) {
        $userInfo = $this->getUserInfo();
        $response->addDataSet(new DataSet('userInfo', $userInfo));
    }
    
    private function getUserInfo(){
        $query = "SELECT O.CDOPERADOR, O.NMOPERADOR, O.NMACESSO, O.NRORG, O.NRORGTRAB, O.NRORGSELECIONADA, O.CDFILIALSELECIONADA, F.NMFILIAL
                  FROM OPERADOR O
                  INNER JOIN FILIAL F ON F.CDFILIAL = O.CDFILIALSELECIONADA
                  WHERE O.CDOPERADOR = :CDOPERADOR";
        $params = array(
         "CDOPERADOR"  => $this->environment->getCdOperador()
        );
        $data = $this->connection->fetchAll($query, $params);
        return $data[0];
    }
    
    /*
    public function logout(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $project    = isset($row['PROJECT'])    ? $row['PROJECT']       : $this->productId;
            $eraseHash  = isset($row['ERASE_HASH']) ? $row['ERASE_HASH']    : false;
            $hash       = isset($row['HASH'])       ? $row['HASH']          : null;

            $this->loginService->logout(
                $project,
                $eraseHash,
                $hash
            );

            $response->addDataSet(new DataSet('system', array('success' => true)));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    */

}