<?php
namespace PME\Controllers\Medicao;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class EfetivosController extends \PME\Services\TeknisaCrud {
  protected $crudService;
	protected $parameterBag;
	protected $environment;
	protected $dataSourceName;
	protected $connection;
  protected $lista;
  protected $newCode;
  protected $numLista;

  public function __construct($crudService, $parameterBag, $connection, $environment, $lista, $newCode) {
    parent::__construct($crudService, $parameterBag, $environment);
    $this->connection  = $connection;
    $this->environment = $environment;
    $this->lista       = $lista;
    $this->newCode     = $newCode;
	}
	
	public function verificaExisteRetirada($row) {
	  if(empty($row['NRCARDPME'])) $row['NRCARDPME'] = null;
    $query = "SELECT SUM(NRCARDAPIO) AS CARDAPIO FROM RETIRADA WHERE CDFILIAL = :CDFILIAL AND NRCARDAPIO = :NRCARDAPIO";
    $params = array(
      'CDFILIAL'   => $row['CDFILIAL'],
      'NRCARDAPIO' => $row['NRCARDPME']
    );
		$stmt = $this->connection->prepare($query);
		$stmt->execute($params);
		return $stmt->fetchAll();
  }
  
  public function verificaExisteFaturamento($row) {
    if(empty($row['DTCARDIAPME'])) $row['DTCARDIAPME'] = null;
    $query = "SELECT FAT FROM (SELECT COUNT(*) AS FAT FROM EFETCLIE WHERE CDFILIAL   = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND DTEFETCLIE = :DTEFETCLIE)";
    $params = array(
      'CDFILIAL'   => $row['CDFILCENTPME'],
      'CDSERVFILI' => $row['CDSERVFILI'],
      'DTEFETCLIE' => $row['DTCARDIAPME']
    );
		$stmt = $this->connection->prepare($query);
		$stmt->execute($params);
		return $stmt->fetchAll();
  }
  
  public function verificaExisteCardia($row){
    $query = "SELECT CARDIAPME.CDSERVFILI FROM CARDIAPME, SERVICO WHERE SERVICO.CDFILIAL = :P_CDFILIAL AND SERVICO.CDFILIAL =  CARDIAPME.CDFILIAL AND SERVICO.CDSERVFILI =  CARDIAPME.CDSERVFILI AND CDFILCENTPME = :P_CDFILCENTPME AND DTCARDIAPME = :P_DTCARDIAPME AND SERVICO.CDSERVFILI = :P_CDSERVFILI AND NRCARDPME = :P_NRCARDPME";
    $params = array(
      "P_CDFILIAL"     => $row['CDFILIAL'],
      "P_NRCARDPME"    => $row['NRCARDPME'],
      "P_CDFILCENTPME" => $row['CDFILCENTPME'],
      "P_CDSERVFILI"   => $row['CDSERVFILI'],
      "P_DTCARDIAPME"  => $row['DTCARDIAPME']
    );
    $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
		return $stmt->fetchAll();
  }
  
	public function verificaExisteCardapioFiltro(DTO\Request $request, DTO\Response $response) {
	  $row = $request->getRow();
	  $query = "SELECT DISTINCT CARDIAPME.CDFILIAL, CARDIAPME.DTCARDIAPME, CARDIAPME.CDSERVFILI, SERVICO.NMSERVICO, CARDIAPME.QTCOMEFECPME, CARDIAPME.QTCLIEFECPME, CARDIAPME.QTFUNEFECPME, CARDIAPME.QTDOAEFECPME, CARDIAPME.QTCOMESTCPME, CARDIAPME.QTCLIESTCPME, CARDIAPME.QTFUNESTCPME, CARDIAPME.QTDOAESTCPME, CARDIAPME.NRCARDPME, CARDIAPME.NRCARDIAPME, CARDIAPME.CDFILCENTPME FROM CARDAPIOPME, CARDIAPME, SERVICO, SERVTIPOCARD WHERE CARDAPIOPME.CDFILCENTPME = :P_CDFILCENTPME AND CARDAPIOPME.CDTIPOCARD = :P_CDTIPOCARD AND CARDAPIOPME.CDFILCENTPME = CARDIAPME.CDFILCENTPME AND CARDAPIOPME.NRCARDPME = CARDIAPME.NRCARDPME AND :P_DTCARDIAPME BETWEEN CARDAPIOPME.DTINICARDPME AND CARDAPIOPME.DTFINCARDPME AND SERVTIPOCARD.CDSERVFILI = CARDIAPME.CDSERVFILI AND SERVTIPOCARD.CDFILCENTPME = CARDIAPME.CDFILCENTPME AND SERVICO.CDFILIAL = CARDIAPME.CDFILIAL AND SERVICO.CDSERVFILI = CARDIAPME.CDSERVFILI AND SERVICO.IDTIPOSERV = 'M' ORDER BY CARDIAPME.CDFILIAL, CARDIAPME.CDSERVFILI";
	  
	  $params = array(
	    "P_CDFILCENTPME" => $row['UNIDADE']['CDFILCENTPME'],
	    "P_DTCARDIAPME" => $row['DTCARDIAPME'],
	    "P_CDTIPOCARD" => $row['TIPOCARD']
	  );
	  
	  $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
    $result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
	}
  
  public function insertCardiapme($row){
    try {
      $query = "INSERT INTO CARDIAPME (CDFILIAL, DTCARDIAPME, CDSERVFILI, QTCOMEFECPME, QTCLIEFECPME, QTFUNEFECPME, QTDOAEFECPME, QTCOMESTCPME, QTCLIESTCPME, QTFUNESTCPME, QTDOAESTCPME, NRCARDPME, NRCARDIAPME, CDFILCENTPME) VALUES (:CDFILIAL, :DTCARDIAPME, :CDSERVFILI, NVL(:QTCOMEFECPME, 0), NVL(:QTCLIEFECPME, 0), NVL(:QTFUNEFECPME, 0), NVL(:QTDOAEFECPME, 0), 0, 0, 0, 0, :NRCARDPME, :NRCARDIAPME, :CDFILCENTPME)";
      $params = array(
        "CDFILIAL"     => $row['CDFILIAL'],
        "DTCARDIAPME"  => $row['DTCARDIAPME'],
        "CDSERVFILI"   => $row['CDSERVFILI'],
        "QTCOMEFECPME" => $row['QTCOMEFECPME'],
        "QTCLIEFECPME" => $row['QTCLIEFECPME'],
        "QTFUNEFECPME" => $row['QTFUNEFECPME'],
        "QTDOAEFECPME" => $row['QTDOAEFECPME'],
        "NRCARDPME"    => $row['NRCARDPME'],
        "NRCARDIAPME"  => $row['NRCARDIAPME'],
        "CDFILCENTPME" => $row['CDFILCENTPME']
      );
      $stmt = $this->connection->prepare($query);
      $stmt->execute($params);
	  } catch (\Exception $e) {
	    throw new $e;
	  }
  }
  
	public function saveServEfetivos(DTO\Request $request, DTO\Response $response) {
    $rows = $request->getRow();
    try {
      $this->connection->beginTransaction();
      
      $msg = array('message' => 'pme::salvoComSucesso', 'type' => 'success');
      
      foreach($rows as $row) {
        $row['NRORG'] = $this->environment->getNrOrg();
        if(empty($row['QTDOAEFECPME'])) $row['QTDOAEFECPME'] = 0;
        if(empty($row['QTCLIEFECPME'])) $row['QTCLIEFECPME'] = 0;
        if(empty($row['QTFUNEFECPME'])) $row['QTFUNEFECPME'] = 0;
        
        $totalQtcomefecpme = $row['QTCLIEFECPME'] + $row['QTFUNEFECPME'] + $row['QTDOAEFECPME'];
  	    $row['QTCOMEFECPME'] = $totalQtcomefecpme;
        
        $existeRetirada = $this->verificaExisteRetirada($row);
        if(!empty($existeRetirada) || $existeRetirada[0]['CARDAPIO'] !== NULL) {
          if($this->verificaExisteFaturamento($row)[0]['FAT'] == 0) {
            if($row['NRCARDPME'] == null) $row['NRCARDPME'] = $this->retornaCardapio($row);
            
            if(!empty($row['NRCARDPME'])) {
              if($row['__is_new']){
              
                if(empty($this->verificaExisteCardia($row))){
                  $row['NRCARDIAPME'] = $this->newCode->getCodeByTableName('CARDIAPME'.$row['CDFILCENTPME'].$row['NRCARDPME'], $row['NRORG']);
                  $this->insertCardiapme($row);
                } else {
                  $clientes = $this->verificaExisteEfetPorCliente($row);
                  if(sizeof($clientes) == 1) {
                    $this->updateCardPme($row);
                    $this->updateEfetCentral($row);
                    foreach($clientes as $cliente){
                      if($cliente['DTEFETCLPME']) {
                        $this->updateEfetPorCliente($row, $cliente);
                      } else {
                        $this->insertEfetPorCliente($row, $cliente);
                      }
                    }
                  } else if (sizeof($clientes) > 1) {
                    $this->updateCardPme($row);
                    $this->updateEfetCentral($row);
                    $msg = array('message' => 'pme::qtdTotalEfetClie', 'type' => 'warning');
                  } else {
                    $this->updateCardPme($row);
                    $this->updateEfetCentral($row);
                  }
                }
              } else {
                $clientes = $this->verificaExisteEfetPorCliente($row);
                if(sizeof($clientes) == 1) {
                  $this->updateCardPme($row);
                  $this->updateEfetCentral($row);
                  foreach($clientes as $cliente){
                    if($cliente['DTEFETCLPME']) {
                      $this->updateEfetPorCliente($row, $cliente);
                    } else {
                      $this->insertEfetPorCliente($row, $cliente);
                    }
                  }
                  $msg = array('message' => 'pme::salvoComSucesso', 'type' => 'success');
                } else if (sizeof($clientes) > 1) {
                  $this->updateCardPme($row);
                  $this->updateEfetCentral($row);
                  $msg = array('message' => 'pme::qtdTotalEfetClie', 'type' => 'warning');
                } else {
                  $this->updateCardPme($row);
                  $this->updateEfetCentral($row);
                  $msg = array('message' => 'pme::salvoComSucesso', 'type' => 'success');
                }
              }
            } else $msg = array('message' => 'pme::naoExisteCardNoDia', 'type' => 'error');
            
          } else {
            $msg = array('message' => 'pme::efetivosFaturados', 'type' => 'error');
          }
        } else {
          $msg = array('message' => 'pme::cardapioNaoPossuiRetirada', 'type' => 'error');
        } 
      }
      
      $this->connection->commit();
      $response->addDataSet(new DataSet("result", $msg));
      
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  private function retornaCardapio($row) {
    $query = "SELECT NRCARDPME FROM CARDAPIOPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND CDTIPOCARD = :P_CDTIPOCARD AND :P_DTCARDIAPME BETWEEN DTINICARDPME AND DTFINCARDPME";
    $params = array(
      "P_CDFILCENTPME" => $row['CDFILCENTPME'],
      "P_CDTIPOCARD"   => $row['CDTIPOCARD'],
      "P_DTCARDIAPME"  => $row['DTCARDIAPME']
    );
    
    return $this->connection->fetchColumn($query, $params);
  }
  
  public function saveClientes(DTO\Request $request, DTO\Response $response) {
    try {
      $rows = $request->getParameters()['rows'];
      $nrorg = $this->environment->getNrOrg();
      $this->connection->beginTransaction();
      $newRows = $rows;
      $this->returnRowsCliente($rows);
      $totalQtcomefecpme = 0;
      $qtdComen = 0;
      $qtdFun = 0;
      $qtdDoa = 0;
      foreach($rows as $row) {
        $qtdComen = $qtdComen + $row['QTCOMEFCLPME'];
        $qtdFun = $qtdFun + $row['QTFUNEFCLPME'];
        $qtdDoa = $qtdDoa + $row['QTDOAEFCLPME'];
      }
      $totalQtcomefecpme = $qtdComen + $qtdFun + $qtdDoa;
      foreach($newRows as $row) {
        $row['NRORG'] = $this->environment->getNrOrg();
        $existeRetirada = $this->verificaExisteRetirada($row);
        if(!empty($existeRetirada) || $existeRetirada[0]['CARDAPIO'] !== NULL) {
          if($this->verificaExisteFaturamento($row)[0]['FAT'] == 0) {
            foreach($newRows as $cliente){
              if($cliente['DTEFETCLPME']) {
                $this->updateCliente($cliente);
              } else {
                $this->insertCliente($cliente);
              }
            }
            $this->updateCardPmeCliente($row, $qtdComen, $qtdFun, $qtdDoa, $totalQtcomefecpme);
            $this->updateEfetCentral($row);
            $msg = array('message' => 'pme::salvoComSucesso', 'type' => 'success');
          } else {
            $msg = array('message' => 'pme::efetivosFaturados', 'type' => 'error');
          }
        } else {
          $msg = array('message' => 'pme::cardapioNaoPossuiRetirada', 'type' => 'error');
        } 
      }
      $this->connection->commit();
      $response->addDataSet(new DataSet("result", $msg));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function returnRowsCliente(&$rows){
    try{
      $query = "SELECT SERVCLIEPME.CDFILIAL, SERVCLIEPME.CDSERVFILI, SERVCLIEPME.CDFILCENTPME, SERVCLIEPME.CDTIPOCARD, TO_CHAR(EFETCLIEPME.DTEFETCLPME, 'DD/MM/YYYY') AS DTEFETCLPME, TO_CHAR(EFETCLIEPME.DTEFETCLPME, 'DD/MM/YYYY') AS DTCARDIAPME, SERVCLIEPME.CDCLIENTE, CLIENTE.NMRAZSOCCLIE, EFETCLIEPME.QTCOMEFCLPME, EFETCLIEPME.QTFUNEFCLPME, EFETCLIEPME.QTDOAEFCLPME, SERVCLIEPME.NRORG FROM CLIENTE, SERVCLIEPME LEFT JOIN EFETCLIEPME ON EFETCLIEPME.DTEFETCLPME = :DTEFETCLIE AND SERVCLIEPME.CDFILCENTPME = EFETCLIEPME.CDFILCENTPME AND SERVCLIEPME.CDTIPOCARD = EFETCLIEPME.CDTIPOCARD AND SERVCLIEPME.CDFILIAL = EFETCLIEPME.CDFILIAL AND SERVCLIEPME.CDSERVFILI = EFETCLIEPME.CDSERVFILI AND SERVCLIEPME.CDCLIENTE = EFETCLIEPME.CDCLIENTE WHERE SERVCLIEPME.CDFILCENTPME = :CDFILCENTPME AND SERVCLIEPME.CDTIPOCARD = :CDTIPOCARD AND SERVCLIEPME.CDFILIAL = :CDFILIAL AND SERVCLIEPME.CDSERVFILI = :CDSERVFILI AND SERVCLIEPME.CDCLIENTE = CLIENTE.CDCLIENTE AND SERVCLIEPME.NRORG = :NRORG ORDER BY CLIENTE.NMRAZSOCCLIE";
      $params = array (
        "DTEFETCLIE" => $rows[0]['DTEFETCLIE'],
        "CDFILCENTPME" => $rows[0]['CDFILCENTPME'],
        "CDTIPOCARD" => $rows[0]['CDTIPOCARD'],
        "CDFILIAL" => $rows[0]['CDFILIAL'],
        "CDSERVFILI" => $rows[0]['CDSERVFILI'],
        "NRORG" => $rows[0]['NRORG']
      );
      $data = $this->connection->fetchAll($query, $params);
      foreach ($data as $row){
        $valid = true;
        foreach ($rows as $editable) {
          if ($editable['CDCLIENTE'] == $row['CDCLIENTE']) {
            $valid = false;  
          }  
        }
        if ($valid) {
          array_push($rows, $row);
        }
      }
    } catch (\Exception $e) {
	    throw new $e;
	  }
  }
	
  private function updateCardPme($row) {
    try {
	    $update = "UPDATE CARDIAPME SET QTCOMEFECPME = :QTCOMEFECPME, QTCLIEFECPME = :QTCLIEFECPME, QTFUNEFECPME = :QTFUNEFECPME, QTDOAEFECPME = :QTDOAEFECPME, QTCOMESTCPME = :QTCOMESTCPME, QTCLIESTCPME = :QTCLIESTCPME, QTFUNESTCPME = :QTFUNESTCPME, QTDOAESTCPME = :QTDOAESTCPME WHERE CDFILCENTPME = :OLD_CDFILCENTPME AND NRCARDPME = :OLD_NRCARDPME AND NRCARDIAPME = :OLD_NRCARDIAPME AND CDFILIAL = :OLD_CDFILIAL AND CDSERVFILI = :OLD_CDSERVFILI";
	    $params = array(
        "QTCOMEFECPME"     => $row['QTCOMEFECPME'],
        "QTCLIEFECPME"     => $row['QTCLIEFECPME'],
        "QTFUNEFECPME"     => $row['QTFUNEFECPME'],
        "QTDOAEFECPME"     => $row['QTDOAEFECPME'],
        "QTCOMESTCPME"     => $row['QTCOMESTCPME'],
        "QTCLIESTCPME"     => $row['QTCLIESTCPME'],
        "QTFUNESTCPME"     => $row['QTFUNESTCPME'],
        "QTDOAESTCPME"     => $row['QTDOAESTCPME'],
        "OLD_CDFILCENTPME" => $row['CDFILCENTPME'],
        "OLD_NRCARDPME"    => $row['NRCARDPME'],
        "OLD_NRCARDIAPME"  => $row['NRCARDIAPME'],
        "OLD_CDFILIAL"     => $row['CDFILIAL'],
        "OLD_CDSERVFILI"   => $row['CDSERVFILI']
      );
      $stmt = $this->connection->prepare($update);
      $stmt->execute($params);
	  } catch (\Exception $e) {
	    throw new $e;
	  }
	}
	
	private function updateCardPmeCliente($row, $qtdComen, $qtdFun, $qtdDoa, $totalQtcomefecpme) {
      $update = "UPDATE CARDIAPME SET QTCOMEFECPME = :QTCOMEFECPME, QTCLIEFECPME = :QTCLIEFECPME, QTFUNEFECPME = :QTFUNEFECPME, QTDOAEFECPME = :QTDOAEFECPME WHERE CDFILCENTPME = :OLD_CDFILCENTPME AND NRCARDPME = :OLD_NRCARDPME AND NRCARDIAPME = :OLD_NRCARDIAPME AND CDFILIAL = :OLD_CDFILIAL AND CDSERVFILI = :OLD_CDSERVFILI";
	    $params = array(
        "QTCOMEFECPME"     => $totalQtcomefecpme,
        "QTCLIEFECPME"     => $qtdComen,
        "QTFUNEFECPME"     => $qtdFun,
        "QTDOAEFECPME"     => $qtdDoa,
        // "QTCOMESTCPME"     => empty($row['QTCOMEFECPME']) ? 0 : $row['QTCOMEFECPME'],
        // "QTCLIESTCPME"     => empty($row['QTCLIEFECPME']) ? 0 : $row['QTCLIEFECPME'],
        // "QTFUNESTCPME"     => empty($row['QTFUNEFECPME']) ? 0 : $row['QTFUNEFECPME'],
        // "QTDOAESTCPME"     => empty($row['QTDOAEFECPME']) ? 0 : $row['QTDOAEFECPME'],
        "OLD_CDFILCENTPME" => $row['CDFILCENTPME'],
        "OLD_NRCARDPME"    => $row['NRCARDPME'],
        "OLD_NRCARDIAPME"  => $row['NRCARDIAPME'],
        "OLD_CDFILIAL"     => $row['CDFILIAL'],
        "OLD_CDSERVFILI"   => $row['CDSERVFILI']
      );
      $stmt = $this->connection->prepare($update);
      $stmt->execute($params);
	}

	private function updateEfetCentral($row) {
    try {
	    $update = "UPDATE CARDIAPME SET QTCOMEFECPME = (SELECT SUM(QTCOMEFECPME) FROM CARDIAPME WHERE (CDFILCENTPME = :P_CDFILCENTPME) AND (NRCARDPME    = :P_NRCARDPME)    AND (NRCARDIAPME = :P_NRCARDIAPME) AND (DTCARDIAPME  = :P_DTCARDIAPME)  AND (CDFILCENTPME <> CDFILIAL)       AND (IDCARDESPEC  <> 'S')), QTCLIEFECPME = (SELECT SUM(QTCLIEFECPME) FROM CARDIAPME WHERE (CDFILCENTPME = :P_CDFILCENTPME) AND (NRCARDPME    = :P_NRCARDPME)    AND (NRCARDIAPME = :P_NRCARDIAPME) AND (DTCARDIAPME  = :P_DTCARDIAPME)  AND (CDFILCENTPME <> CDFILIAL)       AND (IDCARDESPEC  <> 'S')), QTFUNEFECPME = (SELECT SUM(QTFUNEFECPME) FROM CARDIAPME WHERE (CDFILCENTPME = :P_CDFILCENTPME) AND (NRCARDPME    = :P_NRCARDPME)    AND (NRCARDIAPME = :P_NRCARDIAPME) AND (DTCARDIAPME  = :P_DTCARDIAPME)  AND (CDFILCENTPME <> CDFILIAL)       AND (IDCARDESPEC  <> 'S')), QTDOAEFECPME = (SELECT SUM(QTDOAEFECPME) FROM CARDIAPME WHERE (CDFILCENTPME = :P_CDFILCENTPME) AND (NRCARDPME    = :P_NRCARDPME)    AND (NRCARDIAPME = :P_NRCARDIAPME) AND (DTCARDIAPME  = :P_DTCARDIAPME)  AND (CDFILCENTPME <> CDFILIAL)       AND (IDCARDESPEC  <> 'S')) WHERE (CDFILCENTPME = :P_CDFILCENTPME) AND (NRCARDPME    = :P_NRCARDPME)    AND (NRCARDIAPME = :P_NRCARDIAPME) AND (DTCARDIAPME  = :P_DTCARDIAPME)  AND (CDFILCENTPME = CDFILIAL)        AND (IDCARDESPEC  <> 'S')";
	    $params = array(
        "P_CDFILCENTPME" => $row['CDFILCENTPME'],
        "P_NRCARDPME"    => $row['NRCARDPME'],
        "P_NRCARDIAPME"  => $row['NRCARDIAPME'],
        "P_DTCARDIAPME"  => $row['DTCARDIAPME']
      );
      $stmt = $this->connection->prepare($update);
      $stmt->execute($params);
	  } catch (\Exception $e) {
	    throw new $e;
	  }
	}
	
	public function verificaExisteEfetPorCliente($row, $nrlistaUnidade = null) {
    $query = "SELECT SERVCLIEPME.CDFILIAL, SERVCLIEPME.CDSERVFILI, TO_CHAR(EFETCLIEPME.DTEFETCLPME, 'DD/MM/YYYY') AS DTEFETCLPME, SERVCLIEPME.CDCLIENTE, CLIENTE.NMRAZSOCCLIE, EFETCLIEPME.QTCOMEFCLPME, EFETCLIEPME.QTFUNEFCLPME, EFETCLIEPME.QTDOAEFCLPME, SERVCLIEPME.NRORG FROM CLIENTE, SERVCLIEPME LEFT JOIN EFETCLIEPME ON EFETCLIEPME.DTEFETCLPME = :DTEFETCLIE AND SERVCLIEPME.CDFILCENTPME = EFETCLIEPME.CDFILCENTPME AND SERVCLIEPME.CDTIPOCARD = EFETCLIEPME.CDTIPOCARD AND SERVCLIEPME.CDFILIAL = EFETCLIEPME.CDFILIAL AND SERVCLIEPME.CDSERVFILI = EFETCLIEPME.CDSERVFILI AND SERVCLIEPME.CDCLIENTE = EFETCLIEPME.CDCLIENTE WHERE SERVCLIEPME.CDFILCENTPME = :CDFILCENTPME AND SERVCLIEPME.CDTIPOCARD = :CDTIPOCARD AND ( (:NRLISTA IS NOT NULL AND EXISTS ( SELECT CDLISTA FROM LISTA WHERE NRLISTA = :NRLISTA AND SERVCLIEPME.CDFILIAL = CDLISTA )) OR (:NRLISTA IS NULL AND SERVCLIEPME.CDFILIAL = :CDFILIAL)) AND SERVCLIEPME.CDSERVFILI = :CDSERVFILI AND SERVCLIEPME.CDCLIENTE = CLIENTE.CDCLIENTE ORDER BY CLIENTE.NMRAZSOCCLIE";
    $params = array(
      'DTEFETCLIE'   => $row['DTCARDIAPME'],
      'CDFILCENTPME' => $row['CDFILCENTPME'],
      'CDTIPOCARD'   => $row['CDTIPOCARD'],
      'CDFILIAL'     => $row['CDFILIAL'],
      'CDSERVFILI'   => $row['CDSERVFILI'],
      'NRLISTA'      => $nrlistaUnidade
    );
		$stmt = $this->connection->prepare($query);
		$stmt->execute($params);
		return $stmt->fetchAll();
  }
  
  private function insertEfetPorCliente($row, $cliente) {
    try {
      $query = "INSERT INTO EFETCLIEPME (CDFILCENTPME, CDFILIAL, CDSERVFILI, DTEFETCLPME, CDCLIENTE, CDTIPOCARD, QTCOMEFCLPME, QTFUNEFCLPME, QTDOAEFCLPME) VALUES (:CDFILCENTPME, :CDFILIAL, :CDSERVFILI, :DTEFETCLPME, :CDCLIENTE, :CDTIPOCARD, :QTCOMEFCLPME, :QTFUNEFCLPME, :QTDOAEFCLPME)";
  
      $params = array(
        'CDFILCENTPME' => $row['CDFILCENTPME'],
        'CDFILIAL'     => $row['CDFILIAL'],
        'CDSERVFILI'   => $row['CDSERVFILI'],
        'DTEFETCLPME'  => $row['DTCARDIAPME'],
        'CDCLIENTE'    => $cliente['CDCLIENTE'],
        'CDTIPOCARD'   => $row['CDTIPOCARD'],
        'QTCOMEFCLPME' => $row['QTCLIEFECPME'],
        'QTFUNEFCLPME' => $row['QTFUNEFECPME'],
        'QTDOAEFCLPME' => $row['QTDOAEFECPME']
      );
      $stmt = $this->connection->prepare($query);
    	$stmt->execute($params);
	  } catch (\Exception $e) {
	    throw new $e;
	  }
  }
  
  private function updateEfetPorCliente($row, $cliente) {
    try {
  	  $query = "UPDATE EFETCLIEPME SET QTCOMEFCLPME = :QTCOMEFCLPME, QTFUNEFCLPME = :QTFUNEFCLPME, QTDOAEFCLPME = :QTDOAEFCLPME WHERE CDFILCENTPME = :OLD_CDFILCENTPME AND CDTIPOCARD = :OLD_CDTIPOCARD AND CDFILIAL = :OLD_CDFILIAL AND CDSERVFILI = :OLD_CDSERVFILI AND CDCLIENTE = :OLD_CDCLIENTE AND DTEFETCLPME = :OLD_DTEFETCLPME";
	    $params = array(
        "QTCOMEFCLPME"     => $row['QTCLIEFECPME'],
        "QTFUNEFCLPME"     => $row['QTFUNEFECPME'],
        "QTDOAEFCLPME"     => $row['QTDOAEFECPME'],
        "OLD_CDFILCENTPME" => $row['CDFILCENTPME'],
        "OLD_CDTIPOCARD"   => $row['CDTIPOCARD'],
        "OLD_CDFILIAL"     => $row['CDFILIAL'],
        "OLD_CDSERVFILI"   => $row['CDSERVFILI'],
        "OLD_CDCLIENTE"    => $cliente['CDCLIENTE'],
        "OLD_DTEFETCLPME"  => $row['DTCARDIAPME']
      );
      $stmt = $this->connection->prepare($query);
      $stmt->execute($params);
    } catch (\Exception $e) {
	    throw new $e;
	  }
	}
  
  private function insertCliente($row) {
    try {
      $query = "INSERT INTO EFETCLIEPME (CDFILCENTPME, CDFILIAL, CDSERVFILI, DTEFETCLPME, CDCLIENTE, CDTIPOCARD, QTCOMEFCLPME, QTFUNEFCLPME, QTDOAEFCLPME) VALUES (:CDFILCENTPME, :CDFILIAL, :CDSERVFILI, :DTEFETCLPME, :CDCLIENTE, :CDTIPOCARD, :QTCOMEFCLPME, :QTFUNEFCLPME, :QTDOAEFCLPME)";
  
      $params = array(
        'CDFILCENTPME' => $row['CDFILCENTPME'],
        'CDFILIAL'     => $row['CDFILIAL'],
        'CDSERVFILI'   => $row['CDSERVFILI'],
        'DTEFETCLPME'  => $row['DTCARDIAPME'],
        'CDCLIENTE'    => $row['CDCLIENTE'],
        'CDTIPOCARD'   => $row['CDTIPOCARD'],
        'QTCOMEFCLPME' => $row['QTCOMEFCLPME'],
        'QTFUNEFCLPME' => $row['QTFUNEFCLPME'],
        'QTDOAEFCLPME' => $row['QTDOAEFCLPME']
      );
      $stmt = $this->connection->prepare($query);
    	$stmt->execute($params);
	  } catch (\Exception $e) {
	    throw new $e;
	  }
  }
	
	private function updateCliente($row) {
    try {
  	  $query = "UPDATE EFETCLIEPME SET QTCOMEFCLPME = :QTCOMEFCLPME, QTFUNEFCLPME = :QTFUNEFCLPME, QTDOAEFCLPME = :QTDOAEFCLPME WHERE CDFILCENTPME = :OLD_CDFILCENTPME AND CDTIPOCARD = :OLD_CDTIPOCARD AND CDFILIAL = :OLD_CDFILIAL AND CDSERVFILI = :OLD_CDSERVFILI AND CDCLIENTE = :OLD_CDCLIENTE AND DTEFETCLPME = :OLD_DTEFETCLPME";
	    $params = array(
        "QTCOMEFCLPME"     => $row['QTCOMEFCLPME'],
        "QTFUNEFCLPME"     => $row['QTFUNEFCLPME'],
        "QTDOAEFCLPME"     => $row['QTDOAEFCLPME'],
        "OLD_CDFILCENTPME" => $row['CDFILCENTPME'],
        "OLD_CDTIPOCARD"   => $row['CDTIPOCARD'],
        "OLD_CDFILIAL"     => $row['CDFILIAL'],
        "OLD_CDSERVFILI"   => $row['CDSERVFILI'],
        "OLD_CDCLIENTE"    => $row['CDCLIENTE'],
        "OLD_DTEFETCLPME"  => $row['DTCARDIAPME']
      );
      $stmt = $this->connection->prepare($query);
      $stmt->execute($params);
    } catch (\Exception $e) {
	    throw new $e;
	  }
	}
	
	private function updateEfetClienteSave($row) {
    try {
  	  $query = "UPDATE EFETCLIEPME SET QTCOMEFCLPME = :QTCOMEFCLPME, QTFUNEFCLPME = :QTFUNEFCLPME, QTDOAEFCLPME = :QTDOAEFCLPME WHERE CDFILCENTPME = :OLD_CDFILCENTPME AND CDTIPOCARD = :OLD_CDTIPOCARD AND CDFILIAL = :OLD_CDFILIAL AND CDSERVFILI = :OLD_CDSERVFILI AND CDCLIENTE = :OLD_CDCLIENTE AND DTEFETCLPME = :OLD_DTEFETCLPME";
	    $params = array(
        "QTCOMEFCLPME"     => $row['QTCOMEFCLPME'],
        "QTFUNEFCLPME"     => $row['QTFUNEFCLPME'],
        "QTDOAEFCLPME"     => $row['QTDOAEFCLPME'],
        "OLD_CDFILCENTPME" => $row['CDFILCENTPME'],
        "OLD_CDTIPOCARD"   => $row['CDTIPOCARD'],
        "OLD_CDFILIAL"     => $row['CDFILIAL'],
        "OLD_CDSERVFILI"   => $row['CDSERVFILI'],
        "OLD_CDCLIENTE"    => $row['CDCLIENTE'],
        "OLD_DTEFETCLPME"  => $row['DTCARDIAPME']
      );
      $stmt = $this->connection->prepare($query);
      $stmt->execute($params);
    } catch (\Exception $e) {
	    throw new $e;
	  }
	}
	
	public function buscaUnidade(DTO\Request $request, DTO\Response $response) {
	  $rows = $request->getRow();
    $nrorg = $this->environment->getNrOrg();
    $servico = '';
    
    foreach($rows as $row){
      $servico .= $row['CDSERVFILI']."_";
    }
    
	  $row['NRLISTA'] = $this->lista->geraLista($servico, $nrorg);
	  $this->resultadoLista = $row['NRLISTA'];

	  $query = "SELECT DISTINCT 'N' IDMARCADO, F.CDFILIAL, F.NMFILIAL FROM FILIAL F, SERVTIPOCARD ST, SERVICO SE, FILITIPOCARD FT, (SELECT CDLISTA FROM LISTA WHERE NRLISTA = :P_NRLISTA) LS WHERE F.CDFILCENTPME = :P_CDFILCENTPME AND ST.CDFILCENTPME = F.CDFILCENTPME AND FT.CDFILCENTPME = F.CDFILCENTPME AND ST.CDTIPOCARD = :P_CDTIPOCARD AND ST.CDTIPOCARD = FT.CDTIPOCARD AND (ST.CDSERVFILI = LS.CDLISTA) AND ST.CDSERVFILI = SE.CDSERVFILI AND FT.IDTCARATFILI = 'S' AND SE.CDFILIAL = F.CDFILIAL AND FT.CDFILIAL = F.CDFILIAL AND F.CDFILIAL <> :P_CDFILIAL ORDER BY F.CDFILIAL";
	  $params = array(
      "P_CDFILIAL"     => $row["CDFILIAL"],
      "P_NRLISTA"      => $row['NRLISTA'],
      "P_CDFILCENTPME" => $row['CDFILCENTPME'],
      "P_CDTIPOCARD"   => $row['CDTIPOCARD']
    );
    $stmt = $this->connection->prepare($query);
		$stmt->execute($params);
	  $result = $stmt->fetchAll();
	  $response->addDataSet(new DataSet("result", ["data"=> $result, "nrlista" => $row['NRLISTA']]));
	}
	
	public function exportEfetivos(DTO\Request $request, DTO\Response $response){
	  try {
	    $rows = $request->getRow();
	    $nrorg = $this->environment->getNrOrg();
	    $unidade = '';
	    
      if(!empty($rows)) $unidade = implode("_", $rows[0]['unidade']);
      
	    $this->numLista = $this->lista->geraLista($unidade, $nrorg);
	    $this->connection->beginTransaction();
	    foreach($rows as $row){
    	  $params = array(
    	    "P_QTCOMEFECPME" => $row['QTCOMEFECPME'],
    	    "P_QTCLIEFECPME" => $row['QTCLIEFECPME'],
    	    "P_QTFUNEFECPME" => $row['QTFUNEFECPME'],
    	    "P_QTDOAEFECPME" => $row['QTDOAEFECPME'],
    	    "P_CDFILCENTPME" => $row['CDFILCENTPME'],
    	    "P_NRCARDPME"    => $row['NRCARDPME'],
    	    "P_CDSERVFILI"   => $row['CDSERVFILI'],
    	    "P_DTCARDIAPME"  => $row['DTCARDIADEST'],
    	    "P_NRLISTA"      => $this->numLista
    	  );
    	 
  	    $query = "UPDATE CARDIAPME SET QTCOMEFECPME = :P_QTCOMEFECPME, QTCLIEFECPME = :P_QTCLIEFECPME, QTFUNEFECPME = :P_QTFUNEFECPME, QTDOAEFECPME = :P_QTDOAEFECPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME    = :P_NRCARDPME    AND CDSERVFILI   = :P_CDSERVFILI   AND DTCARDIAPME  = :P_DTCARDIAPME AND EXISTS (SELECT CDLISTA FROM LISTA WHERE NRLISTA = :P_NRLISTA AND CDFILIAL = CDLISTA)";
    	  $stmt = $this->connection->prepare($query);
  			$stmt->execute($params);

  			$this->updateCliDest($row);
  			
	    }
		  $this->deleteListaServico($request, $response);
		  $this->deleteListaUnidade($request, $response);
		  $this->connection->commit();
	  } catch(\Exception $e){
	    $this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
	  }
	}
	
	private function updateCliDest($row) {
	  $query = "SELECT CDCLIENTE, QTCOMEFCLPME, QTFUNEFCLPME, QTDOAEFCLPME  FROM EFETCLIEPME  WHERE CDFILCENTPME = :CDFILCENTPME AND CDTIPOCARD = :CDTIPOCARD AND CDFILIAL = :CDFILIALORIGEM AND CDSERVFILI = :CDSERVFILI AND DTEFETCLPME = :DTORIGEM";
    $params = array (
      "CDFILCENTPME"  => $row['CDFILCENTPME'],
      "CDTIPOCARD"    => $row['CDTIPOCARD'],
      "CDFILIALORIGEM"=> $row['CDFILIAL'],
      "CDSERVFILI"    => $row['CDSERVFILI'],
      "DTORIGEM"      => $row['DTCARDIAPME']
    );
    $cliOri = $this->connection->fetchAll($query, $params);
    
    $filtro = $row;
    $filtro['DTCARDIAPME'] = $row['DTCARDIADEST'];
    $cliDest = $this->verificaExisteEfetPorCliente($filtro, $this->numLista);
    foreach($cliOri as $ori) {
      foreach($cliDest as $dest) {
        if($ori['CDCLIENTE'] == $dest['CDCLIENTE']) {
          
          $params = array(
      	    "QTCOMEFCLPME" => $ori['QTCOMEFCLPME'],
      	    "QTFUNEFCLPME" => $ori['QTFUNEFCLPME'],
      	    "QTDOAEFCLPME" => $ori['QTDOAEFCLPME'],
      	    "CDFILCENTPME" => $row['CDFILCENTPME'],
      	    "CDTIPOCARD"   => $row['CDTIPOCARD'],
      	    "CDSERVFILI"   => $row['CDSERVFILI'],
      	    "DTDEST"       => $row['DTCARDIADEST'],
      	    "NRLISTA"      => $this->numLista
      	  );
      	  
          if (!empty($dest['DTEFETCLPME'])) {
            $query = "UPDATE EFETCLIEPME SET QTCOMEFCLPME = :QTCOMEFCLPME, QTFUNEFCLPME = :QTFUNEFCLPME, QTDOAEFCLPME = :QTDOAEFCLPME WHERE CDFILCENTPME = :CDFILCENTPME AND CDTIPOCARD = :CDTIPOCARD AND CDSERVFILI = :CDSERVFILI AND DTEFETCLPME = :DTDEST AND CDCLIENTE = :CDCLIENTE AND EXISTS ( SELECT CDLISTA FROM LISTA WHERE NRLISTA = :NRLISTA AND CDFILIAL = CDLISTA )";  
            $params["CDCLIENTE"] = $ori['CDCLIENTE'];
          } else $query = "INSERT INTO EFETCLIEPME ( CDFILCENTPME, CDFILIAL, CDSERVFILI, CDTIPOCARD, CDCLIENTE, DTEFETCLPME, QTCOMEFCLPME, QTFUNEFCLPME, QTDOAEFCLPME ) SELECT CDFILCENTPME, CDFILIAL, CDSERVFILI, CDTIPOCARD, CDCLIENTE, :DTDEST AS DTEFETCLPME, :QTCOMEFCLPME AS QTCOMEFCLPME, :QTFUNEFCLPME AS QTFUNEFCLPME, :QTDOAEFCLPME AS QTDOAEFCLPME FROM SERVCLIEPME WHERE CDFILCENTPME = :CDFILCENTPME AND CDTIPOCARD = :CDTIPOCARD AND CDSERVFILI = :CDSERVFILI AND EXISTS ( SELECT CDLISTA FROM LISTA WHERE NRLISTA = :NRLISTA AND CDFILIAL = CDLISTA )";
          
      	  $stmt = $this->connection->prepare($query);
    			$stmt->execute($params);  
        }
        
      }
    }
	}
	
	public function deleteListaServico(DTO\Request $request, DTO\Response $response){
	  $rows = $request->getRow();
	  $query = "DELETE FROM LISTA WHERE NRLISTA = :P_NRLISTA";
	  foreach($rows as $row){
  	  $params = array(
        "P_NRLISTA" => $row["nrlista"]
      );
	  }
	  $std = $this->connection->prepare($query);
    $std->execute($params); 
	}
	
	public function deleteListaUnidade(DTO\Request $request, DTO\Response $response){
	  $rows = $request->getRow();
	  $query = "DELETE FROM LISTA WHERE NRLISTA = :P_NRLISTA";
	  foreach($rows as $row){
  	  $params = array(
        "P_NRLISTA" => $this->numLista
      );
	  }
	  $std = $this->connection->prepare($query);
    $std->execute($params); 
	}
}