<?php
namespace Fiscal\Logic\NotaFiscal\Entrada;

/**
 * Created by JetBrains PhpStorm.
 * User: maximiliansilva
 * Date: 16/01/13
 * Time: 14:08
 * To change this template use File | Settings | File Templates.
 */
use \Estoque\Logic\GeralLancamentoEstoque\LancamentoEstoque;
use \Estoque\Logic\AtualizacaoEstoque\AtualizadorDeEstoque;
use Fiscal\Entities\IntgNotafiscal;
use Fiscal\Entities\Notafiscal;
use Fiscal\Logic\ImportacaoAF\ImportadorAF;
use \Fiscal\Logic\NotaFiscal\Exceptions\NFEntradaException;
use Fiscal\Logic\NotaFiscal\ProceduresNF;
use Geral\Entities\Itemproduto;
use Setforms2\Shared\ORM\BasicEntity;
use Setforms2\Shared\Registry;
use Geral\Util\DateUtil;
use Fiscal\Entities\Trabitproimpos;
use Fiscal\Entities\Trabimposnftot;
use Geral\Entities\Itemnotafisc;
use Geral\Entities\Tipooperac;

class NFValidaNotafiscal{
    /* @var $notafiscal \Fiscal\Entities\Notafiscal */
    public $notafiscal;

    public function __construct(\Fiscal\Entities\Notafiscal $notafiscal){
        $this->notafiscal = $notafiscal;
    }

    public function dataIntegracaoAgent2bValida(){
        $tpOperacao = $this->notafiscal->getTipoOperac();

        if (is_object($tpOperacao) && $tpOperacao->getIdcontaboper() === 'S'){
            if ($this->validaDataIntegracao()){
                return true;
            }
            else{
                return NFEntradaException::dataIntegracaoAgente2bInvalida();
            }
        }
        return true;
    }

    public function validaDataIntegracao(){
        $contOpeCtbl = $this->notafiscal->getContopectbl();
        if (is_object($contOpeCtbl)){
            if ($this->notafiscal->getEntrsaidop() === 'E' && $contOpeCtbl->getDtiniintcontdfe()){
                $dtiniintcontdfe = new \Zend_Date($contOpeCtbl->getDtiniintcontdfe());
                $dtfinintcontdfe = new \Zend_Date($contOpeCtbl->getDtfinintcontdfe());
                $dtentrada       = new \Zend_Date($this->notafiscal->getDtentrsaid());
                if ($dtiniintcontdfe->isEarlier($dtentrada) && $dtfinintcontdfe->isLater($dtentrada)){
                    return true;
                }
                else{
                    return false;
                }
            }
            else if ($contOpeCtbl->getDtiniintcontdfs()){
                $dtiniintcontdfs = new \Zend_Date($contOpeCtbl->getDtiniintcontdfs());
                $dtfinintcontdfs = new \Zend_Date($contOpeCtbl->getDtfinintcontdfs());
                $dtentrada       = new \Zend_Date($this->notafiscal->getDtentrsaid());
                if ($dtiniintcontdfs->isEarlier($dtentrada) && $dtfinintcontdfs->isLater($dtentrada)){
                    return true;
                }
                else{
                    return false;
                }
            }
            return true;
        }
        return true;
    }

    public function notaFiscalCompraPossuiAf(){
        $idParambloqnfafil = array(
            'nrorg'        => $this->notafiscal->getNrorg(),
            'cdfilial'     => $this->notafiscal->getCdfilial(),
            'cdtipooper'   => $this->notafiscal->getCdtipooper(),
            'identrsaidop' => $this->notafiscal->getIdentrsaidop());
        /** @var $paramnfaf \Estoque\Entities\Parambloqnfafil */
        $paramnfaf = \Estoque\Entities\Parambloqnfafil::getObject($idParambloqnfafil);

        /** @var $paramcmp \Suprimentos\Entities\Paramcmpprod */
        $paramcmp = \Suprimentos\Entities\Paramcmpprod::getObjectByWhere('NRORG = ? AND CDFILIAL = ? AND IDBLOQNFSAF = ?',
            array(
                $this->notafiscal->getNrorg(),
                $this->notafiscal->getCdfilial(),
                'N'));
        if (is_object($paramnfaf)){
            if ($paramnfaf->getIdexigeaf() === 'S' && empty($paramcmp)){
                return NFEntradaException::notafiscalCompraNaoPossuiAF();
            }
        }
        return true;
    }

    public function lancamentoPermitidoParafornecedor(){
        $id = array(
            'nrorg'      => \Geral\Util\Ambiente::getNrorg(),
            'cdforneced' => $this->notafiscal->getCdforneced());
        /* @var $paramForn \Indeterminado\Entities\Paramforne */
        $paramForn = \Indeterminado\Entities\Paramforne::getObject($id);
        if (is_object($paramForn) && $paramForn->getIdblqnfrelafoaf() === 'S'){
            return NFEntradaException::fornecedorNaoPermitidoLancarNF();
        }
        return true;
    }

    /**
     * Valida a data de entrada esta de acordo com a data de processamento.
     *
     * @return boolean|string
     * @author fernandomatias - 19/04/2012
     */
    public function validaDataEntrada(){
        if (!LancamentoEstoque::validaData($this->notafiscal->getDtentrsaid(), 'entrada',
            \Estoque\Util\Parametros::getDataProcessamento())
        ){

            return NFEntradaException::dataEntradaForaPeriodo();
        }
        return true;
    }

    /**
     * Valida a data de saída esta de acordo com a data de processamento.
     */
    public function validaDataSaida(){
        if (!LancamentoEstoque::validaData($this->notafiscal->getDtentrsaid(), 'saida',
            \Estoque\Util\Parametros::getDataProcessamento())
        ){

            return NFEntradaException::dataSaidaForaPeriodo();
        }
        return true;
    }

    public function verificaExistenciaOcorrenciaAtendimento(){
        /** @var $movocorr  \Qualificacao\Entities\Movocorr */
        $movocorr = \Qualificacao\Entities\Movocorr::getObjectByWhere('NRORG = ? AND NRLANCTONF = ? AND CDFILIALNF = ?',
            array(
                $this->notafiscal->getNrorg(),
                $this->notafiscal->getNrlanctonf(),
                $this->notafiscal->getCdfilial()));

        if (is_object($movocorr)){
            NFEntradaException::existeOcorrenciaAtendimento($movocorr->getNrocorrmov());
        }

        return true;
    }

    public function notaEmitidaAprovada(){
        if ($this->notafiscal->getIdstatus() === 'A' || $this->notafiscal->getIdstatus() === 'E'){
            NFEntradaException::notafiscalEmetidaOuAprovado();
        }
        return true;
    }

    /**
     * Atualiza o estoque e verifica se ao excluir a nota fiscal alguns itens gerariam valores negativos.
     *
     * @return boolean|string
     * @author fernandomatias 19/04/2012
     */
    public function atualizaEstoqueExclusaoNotaFiscal(){
        // @ todo verificar rotina de exclusao de nota
        if (is_object($this->notafiscal->getLanctoestoq())){
            \Fiscal\Logic\NotaFiscal\ProceduresNF::procedureDeletaNF($this->notafiscal);
            AtualizadorDeEstoque::atualizarEstoque($this->notafiscal->getLanctoestoq());
        }
    }

    public function atualizaStatusAF(){
        $itensNota = Itemproduto::findByCdfilialAndNrlanctonfAndNrorg($this->notafiscal->getCdfilial(),
            $this->notafiscal->getNrlanctonf(), $this->notafiscal->getNrorg());
        foreach ($itensNota as $itemproduto){
            $paramsBaixaAF = ImportadorAF::montaParamsBaixaAF($itemproduto, 'E');
            ProceduresNF::callProcedureBaixaAF($paramsBaixaAF);
        }
    }

    public function atualizaStatusXML(){
        $manager = Registry::getEntityManager();

        /** @var $xml IntgNotafiscal */
        $xml = IntgNotafiscal::findOneByNracessonfe($this->notafiscal->getNracessonfe());

        $xml->setIdstatusnfe('N');
        $xml->setIdimportada('N');
        $xml->setCdfilirece(null);
        $xml->setDtutilizaxml(null);
        $xml->setCdfilial(null);
        $xml->setNrlanctonf(null);

        $xml->persist();

        $manager->flush();
    }

    public function deletaImpostosTemporarios(){
        self::deletaImpostosTemporariosItem();
        self::deletaImpostosTemporariosNota();
    }

    public function deletaImpostosTemporariosItem(){
        /** @var $trabitproimpos Trabitproimpos */
        $trabitproimpos = Trabitproimpos::getObjectsByWhere('CDFILIMPOIT = ? AND NRLANCIMPOIT = ? AND NRORG = ?', array(
            $this->notafiscal->getCdfilial(),
            $this->notafiscal->getNrlanctonf(),
            $this->notafiscal->getNrorg()));

        foreach ($trabitproimpos as $imposto){
            $manager = Registry::getEntityManager();

            $imposto->delete();

            $manager->flush();
        }
    }

    public function deletaImpostosTemporariosNota(){
        /** @var $trabimposnftot Trabimposnftot */
        $trabimposnftot = Trabimposnftot::getObjectsByWhere('CDFILIMPTOT = ? AND NRLANCIMPTOT = ? AND NRORG = ?', array(
            $this->notafiscal->getCdfilial(),
            $this->notafiscal->getNrlanctonf(),
            $this->notafiscal->getNrorg()));

        foreach ($trabimposnftot as $imposto){
            $manager = Registry::getEntityManager();

            $imposto->delete();

            $manager->flush();
        }
    }

    public function deletaNota(){
        \Fiscal\Logic\NotaFiscal\ProceduresNF::procedureDeletaNF($this->notafiscal);
    }

    /**
     * Valida se o fornecedor esta ativo.
     *
     * @static
     * @return boolean
     * @author fernandomatias - 24/04/2012
     */
    public function validaFonecedorAtivo(){
        $fornecedor = $this->notafiscal->getForneced();
        if (is_object($fornecedor) && $fornecedor->getIdsitucada() != \Geral\Entities\Fornecedor::ID_FORNECEDOR_ATIVO){
            return NFEntradaException::fornecedorInativo();
        }
        return true;
    }

    /**
     * Valida se o forncedor tem permissao de compra e cotacao.
     *
     * @static
     * @return boolean
     * @author fernandomatias - 25/04/2012
     */
    public function validaPermissaoFornecedor(){
        $fornecedor = $this->notafiscal->getForneced();

        if (!is_object($fornecedor)){
            return true;
        }
        if ($fornecedor->getCdqualif()){
            $qualif = $fornecedor->getQualif();
            if (is_object($qualif)){
                if ($qualif->getIdpercotcmp() === 1 || ($qualif->getIdpercotcmp() === '2') && $fornecedor->getIdfornauto() === 'S'){
                    $idutilval = \Suprimentos\Util\Parametro::getUltilizaDtValidadeQualificacoes();
                    $dtentrada = new \Zend_Date($this->notafiscal->getDtentrsaid()->format('d/m/Y'), 'dd/MM/y');
                    $dtqualif  = new \Zend_Date($fornecedor->getDtvalqualif()->format('d/m/Y'), 'dd/MM/y');
                    $ultdia    = new \Zend_Date($fornecedor->getDtvalqualif()->format('d/m/Y'), 'dd/MM/y');
                    $ultdia->setDay(01)->addMonth(1)->subDay(1);
                    if (($idutilval === 'D' && ($dtqualif->isLater($dtentrada))) || ($idutilval === 'M' && ($dtqualif->isLater($ultdia)))){
                        return true;
                    }
                }
            }
            else if (!$fornecedor->getCdqualif() || $fornecedor->getIdpercmpdfe() === 'S'){
                return true;
            }
        }

        if ((!$fornecedor->getCdqualif()) || ($fornecedor->getIdpercmpdfe() === 'S')){
            return true;
        }

        if (\Fiscal\Util\Parametros::getPermissaoFornedorParaComprasCotacoes() === 'N'){
            return NFEntradaException::permiteFornecedorCotacaoCompra();
        }

        return true;
    }

    /**
     * Valida se o fornecedor utiliza nota fiscal eletronica.
     *
     * @static
     * @return boolean|string
     * @author fernandomatias - 26/04/2012
     */
    public function fornecedorUtilizaIntegracao(){
        $fornecedor = $this->notafiscal->getForneced();

        if (!is_object($fornecedor) && $fornecedor->getIdutiledinf() === 'N'){
            return true;
        }
        else{
            return NFEntradaException::fornecedorUtilizaIntegracao();
        }
    }

    public function validaDataEmissao(){
        $nrdiasbloqdtemi = \Fiscal\Util\Parametros::getNumerosDiasBloqueioDataEmissao();
        $nrdiasavisdtemi = \Fiscal\Util\Parametros::getNumerosDiasAvisosDataEmissao();
        $dtemissao       = new \Zend_Date($this->notafiscal->getDtemissao()->format('d/m/Y'));
        $dtentrsaid      = new \Zend_Date($this->notafiscal->getDtentrsaid()->format('d/m/Y'));
        if ($nrdiasbloqdtemi || $nrdiasbloqdtemi != 0){
            if ($dtemissao->isEarlier($dtentrsaid->subDay($nrdiasbloqdtemi))){
                return NFEntradaException::dataEmissaoBloqueada();
            }
        }

        if ($nrdiasavisdtemi || $nrdiasavisdtemi != 0){
            if ($dtemissao->isEarlier($dtentrsaid->subDay($nrdiasavisdtemi))){
                return NFEntradaException::dataEmissaoForaPeriodo();
            }
        }

        if (DateUtil::comparaDatas($this->notafiscal->getDtentrsaid(), $this->notafiscal->getDtemissao()))
            return NFEntradaException::dataEmissaoMaiorDataEntrada();

        return true;
    }

    /**
     * Verifica se ha inventario pendente.
     *
     * @static
     * @return string
     * @author maximiliansilva 22/01/13
     */
    public function verificaInventarioPendente(){
        $cdalmoxarife = \Estoque\Util\Parametros::getAlmoxarifado();
        $cdlocalestoq = \Estoque\Util\Parametros::getLocalEstoque();

        if (\Estoque\Util\Parametros::getUtilizaLote() === 'S'){
            /* @var $inverotalote \Estoque\Repositories\Inverotalote */
            $inverotalote = \Estoque\Entities\Inverotalote::getRepository();
            $result       = $inverotalote->inventarioLoteOk($this->notafiscal->getCdfilial(), $cdalmoxarife,
                $cdlocalestoq, $this->notafiscal->getDtentrsaid()->format('d/m/Y'));
        }
        else{
            /* @var $inverota \Estoque\Repositories\Inverota */
            $inverota = \Estoque\Entities\Inverota::getRepository();
            $result   = $inverota->inventarioOk($this->notafiscal->getCdfilial(), $cdalmoxarife, $cdlocalestoq,
                $this->notafiscal->getDtentrsaid()->format('d/m/Y'));
        }
        if ($result){
            NFEntradaException::inventarioPendenteFilial();
        }

        return true;
    }

    public function notaFiscalJaCadastrada(){
        if ($this->notafiscal->getIdorigdestnf() == 'F'){
            /** @var $nf Notafiscal */
            $nf = Notafiscal::findOneByCdfornecedAndCdserieentrAndNrnotafiscAndNrorg($this->notafiscal->getCdforneced(),
                $this->notafiscal->getCdserieentr(), $this->notafiscal->getNrnotafisc(), $this->notafiscal->getNrorg());
        }
        elseif ($this->notafiscal->getIdorigdestnf() == 'L'){
            /** @var $nf Notafiscal */
            $nf = Notafiscal::findOneByCdfiliemitAndCdserieentrAndNrnotafiscAndNrorg($this->notafiscal->getCdfiliemit(),
                $this->notafiscal->getCdserieentr(), $this->notafiscal->getNrnotafisc(), $this->notafiscal->getNrorg());
        }
        elseif ($this->notafiscal->getIdorigdestnf() == 'C'){
            /** @var $nf Notafiscal */
            $nf = Notafiscal::findOneByCdclienteAndCdserieentrAndNrnotafiscAndNrorg($this->notafiscal->getCdcliente(),
                $this->notafiscal->getCdserieentr(), $this->notafiscal->getNrnotafisc(), $this->notafiscal->getNrorg());
        }

        if ($nf instanceof Notafiscal)
            NFEntradaException::notaFiscalJaCadastrada();
    }

    public function XMLJaCadastrado(){
        /** @var $xml IntgNotafiscal */
        $xml = IntgNotafiscal::findOneByNracessonfe($this->notafiscal->getNracessonfe());

        if ($xml instanceof IntgNotafiscal && $xml->getIdimportada() == 'S')
            NFEntradaException::XMLJaCadastrado();

        return true;
    }

    public function validaFilialInformadaTransferencia(){
        $filial_logada = \Geral\Util\Ambiente::getCdFilial();
        $cdfilial      = $this->notafiscal->getCdfiliemit();

        if ($filial_logada == $cdfilial)
            NFEntradaException::filiaLogadaIgualFilialInformada();
    }

    public function verificaNumNFPreenchido(){
        $tpOper = $this->notafiscal->getCdtipooper();

        if (!in_array($tpOper, array(
            '08',
            '09',
            '10',
            '12',
            '13')) && $this->notafiscal->getNrnotafisc() == null
        ){
            /* @var $tipoOperacao Tipooperac */
            $tipoOperacao = Tipooperac::findOneByCdtipooperAndNrorgAndIdentrsaidop($tpOper,
                $this->notafiscal->getNrorg(), $this->notafiscal->getIdentrsaidop());

            if ($tipoOperacao->getIdutilemisnf() != 'S')
                return NFEntradaException::numeroNFdeveSerInformado();
        }
    }

    public function verificaSeriePreenchida(){
        $tpOper = $this->notafiscal->getCdtipooper();

        if (!in_array($tpOper, array(
            '09',
            '10',
            '12',
            '13')) && $this->notafiscal->getCdserieentr() == null
        ){
            return NFEntradaException::serieDeveSerInformada();
        }
    }
}
