import {
  Http, Config, Cookie, Loader, Metadata, I18n, IEventParam, IDictionary, dayjs as moment,
} from '@zeedhi/core';
import {
  Modal, ModalService, Image, Row, Form, Text,
} from '@zeedhi/common';

import { PmeCurrencyMask } from '@/config';

export class LoginController {
  private forgotModal!: Modal;

  private email = '';

  private user = '';

  public utilities = Loader.getInstance('Utilities');

  public appController = Loader.getInstance('AppController');

  private get language() {
    return localStorage.getItem('LOGIN_LANGUAGE');
  }

  private passwordModal!: Modal;

  get backgroundStyle(): string {
    return `url(${Config.env.loginBackground})`;
  }

  public loginFail(response: IDictionary) {
    // store.commit('loading', false);
    this.utilities.showAlert('Alert', I18n.translate(`pme::${response.error}`), 'error');
  }

  public setFocusLanguage() {
    const form = Metadata.getInstance<Row>('rowIcons');
    const language = Cookie.getCookie('language');
    const children = form.childrenInstances;
    children.forEach((e: IDictionary) => {
      if (e.component === 'ZdImage') {
        if (e.name === language) e.cssClass = 'icon-focus';
        else e.cssClass = 'icon';
      }
    });
  }
  
  /*
  public loginSuccess({ response }: IDictionary) {
    if (response.data.dataset) {
      if (response.data.dataset.system.EXPIRED_PASSWORD) {
        this.user = response.data.dataset.system.USER_NAME;
        this.email = response.data.dataset.system.USER_EMAIL;
        this.openPasswordModal();
      } else {
        this.setLocalStorageUser(response.data.dataset.system);
        Cookie.setCookie('token', response.data.dataset.system.TOKEN, 86400000);
        router.push('/');
      }
    }
    return true;
  }

  public login({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('dataForm');
    if (form.validate()) {
      this.utilities.activeLoadingButton(component);
      Http.post('/authenticate', { language: this.language, user: form.value.user, password: form.value.password }).then((response) => {
        this.loginSuccess({ response });
      }).catch((error) => {
        this.loginFail({ error });
        this.utilities.disableLoadingButton(component);
      }).finally(() => {
        // this.utilities.disableLoadingButton(component);
      });
    }
  }
  */
  
  public setLocalStorageUser(userData: IDictionary) {
    if (userData) {
      /*
      localStorage.setItem('CDOPERADOR', userData.USER_ID);
      localStorage.setItem('NMOPERADOR', userData.USER_NAME);
      localStorage.setItem('CDFILIAL', userData.CDFILIALSELECIONADA);
      localStorage.setItem('NRORG', userData.NRORGSELECIONADA);
      localStorage.setItem('LANGUAGE', userData.LANGUAGE);
      localStorage.setItem('CDPICTPROD', userData.CDPICTPROD);
      localStorage.setItem('CDPICTPRAT', userData.CDPICTPRAT);
      localStorage.setItem('NMFILIALOGADO', userData.NMFILIAL);
      localStorage.setItem('CDOPERADORLOGADO', userData.CDFILIALSELECIONADA);
      */
        
      Http.post('/userInfo').then((response) => {
        const userInfo = response.data?.dataset?.userInfo;
        if (userInfo) {
          localStorage.setItem('CDFILIALLOGADA', userInfo.CDFILIALSELECIONADA);
          localStorage.setItem('NMFILIALLOGADA', userInfo.NMFILIAL);
          Metadata.getInstance<Text>('cdCentralFooter').text = userInfo.CDFILIALSELECIONADA;
          Metadata.getInstance<Text>('nmCentralFooter').text = userInfo.NMFILIAL;
        }
      });
      
      // aqui está sendo setado a data de processamento no footer da aplicação
      this.utilities.getProcessDate().then((processDate: string) => {
        Metadata.getInstance<Text>('processDate').text = moment(processDate, 'DD/MM/YYYY').format(this.utilities.getFormatLocale());
        localStorage.setItem('DTPROCESSAMENTO', processDate);
      });
  
      // let lang = 'pt';
      const lang = this.utilities.getLanguage();
      const locale: string = this.appController.metaLanguages[lang];
      const currency: string = this.appController.metaCurrency[lang];
      
      I18n.changeLanguage(this.appController.metaLanguages[lang]);
      // if (typeof userData.LANGUAGE === 'string' && userData.LANGUAGE.length >= 2) lang = userData.LANGUAGE.substring(0, 2);
      
      // moment.locale(lang);
      // localStorage.setItem('DATEFORMAT', moment.localeData(lang).longDateFormat('L'));
      localStorage.setItem('DATEFORMAT', this.utilities.getFormatLocale());
      
      Config.displayFormat = this.utilities.getFormatLocale();
  
      (Config.masks.numberMask as PmeCurrencyMask).currencySymbol = this.utilities.getCurrencySymbol(locale, currency);
      localStorage.setItem('CURRENCY', (Config.masks.numberMask as PmeCurrencyMask).currencySymbol);
    }
  }

  // modal
  public requestNewPassword() {
    const formForgotModal = Metadata.getInstance<Form>('formForgotModal');
    if (formForgotModal.validate()) {
      if (this.email && typeof this.email === 'string') {
        Http.post('/requestNewPassword', { email: this.email }).then(() => {
          this.utilities.showAlert('Alert', 'pme::emailRecuperacaoEnviado', 'success');
          this.closeForgotModal();
        }).catch(() => {
          this.utilities.showAlert('Alert', 'pme::usuarioNaoEncontrado', 'error');
        }).finally(() => {
          this.email = '';
        });
      } else {
        this.utilities.showAlert('Alert', 'pme::usuarioNaoEncontrado', 'error');
      }
    }
  }
  /*

  public updatePassword({ component }: IEventParam<Button>) {
    const password = Metadata.getInstance<Password>('password');
    const newPassword = Metadata.getInstance<Password>('newPassword');
    const confirmPassword = Metadata.getInstance<Password>('confirmPassword');

    const row = {
      email: this.email,
      actualPassword: password.value,
      newPassword: newPassword.value,
      confirmPassword: confirmPassword.value,
    };

    if (row.newPassword !== row.confirmPassword) {
      this.utilities.showAlert('Alert', 'pme::senhasNaoCoincidem', 'error');
    } else {
      this.utilities.activeLoadingButton(component);
      Http.post('/updatePassword', row).then(() => {
        this.utilities.showAlert('Alert', 'pme::senhaAlterada', 'success');
        this.authenticate(row);
      }).catch(() => {
        this.utilities.showAlert('Alert', 'pme::senhaNaoAlterada', 'error');
      }).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
      this.closePasswordModal();
    }
  }
  
  private authenticate(row: IDictionary) {
    const user = Metadata.getInstance<TextInput>('user');
    Http.post('/authenticate', { language: this.language, user: user.value, password: row.newPassword }).then((response) => {
      this.loginSuccess({ response });
    }).catch((error) => {
      this.utilities.showAlert('Alert', error, 'error');
    });
  }
  */
  
  public setLanguage(component: IEventParam<Image>) {
    const language = component.component.name;
    const form = Metadata.getInstance<Row>('rowIcons');
    const children = form.childrenInstances;
    children.forEach((e: IDictionary) => {
      if (e.component === 'ZdImage') e.cssClass = 'icon';
    });
    component.component.cssClass = 'icon-focus';
    I18n.changeLanguage(this.appController.metaLanguages[language]);
    Cookie.setCookie('language', language);
  }

  public destroyModals() {
    this.forgotModal.destroy();
    this.passwordModal.destroy();
  }

  public openPasswordModal() {
    this.passwordModal.show();
  }

  public closePasswordModal() {
    this.passwordModal.hide();
  }

  public openForgotModal() {
    this.forgotModal.show();
  }

  public closeForgotModal() {
    this.forgotModal.hide();
  }

  public async createModals() {
    this.forgotModal = ModalService.create({
      name: 'forgotModal',
      cssClass: 'senha-modal',
      children: [
        {
          name: 'header',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'pme::redefinirSuaSenha',
              cssClass: 'zd-theme-font-title',
              style: {
                color: '#00305A',
              },
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdModalCloseButton',
              small: true,
              modalName: 'forgotModal',
            },
          ],
        },
        {
          name: 'formForgotModal',
          component: 'ZdForm',
          children: [
            {
              name: 'email',
              label: 'pme::email',
              component: 'ZdTextInput',
              type: 'Email',
              value: '{{LoginController.email}}',
              appendIcon: 'mdi-email',
              validations: {
                required: {
                  message: 'pme::campoObrigatorio',
                },
              },
            },
            {
              name: 'footer',
              component: 'ZdFooter',
              color: 'transparent',
              padless: true,
              elevation: 0,
              rightSlot: [
                {
                  name: 'enviar',
                  component: 'ZdButton',
                  label: 'pme::redefinirSenha',
                  events: {
                    click: '{{LoginController.requestNewPassword}}',
                  },
                  normal: true,
                },
              ],
            },
          ],
        },
      ],
    });

    this.passwordModal = ModalService.create({
      name: 'passwordModal',
      cssClass: 'senha-modal',
      children: [
        {
          name: 'header',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'pme::redefinirSenha',
              cssClass: 'zd-theme-font-title',
              style: {
                color: '#00305A',
              },
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdModalCloseButton',
              small: true,
              modalName: 'passwordModal',
            },
          ],
        },
        {
          name: 'form',
          component: 'ZdForm',
          children: [
            {
              name: 'user',
              label: 'pme::usuario',
              component: 'ZdTextInput',
              disabled: true,
              value: '{{LoginController.user}}',
              appendIcon: 'mdi-user',
            },
            {
              name: 'newPassword',
              component: 'ZdPassword',
              label: 'pme::novaSenha',
              validations: {
                required: {
                  message: 'pme::campoObrigatorio',
                },
              },
              grid: {
                cols: 6,
              },
            },
            {
              name: 'confirmPassword',
              component: 'ZdPassword',
              label: 'pme::confirmeNovaSenha',
              validations: {
                required: {
                  message: 'pme::campoObrigatorio',
                },
              },
              grid: {
                cols: 6,
              },
            },
            {
              name: 'footer',
              component: 'ZdFooter',
              color: 'transparent',
              padless: true,
              elevation: 0,
              rightSlot: [
                {
                  name: 'confirmar',
                  component: 'ZdButton',
                  label: 'pme::confirmar',
                  events: {
                    click: '{{LoginController.updatePassword}}',
                  },
                  normal: true,
                },
              ],
            },
          ],
        },
      ],
    });
  }
}
