import Vue from 'vue';
import Router from 'vue-router';
import { Config, Http } from '@zeedhi/core';
import { ZdPage } from '@zeedhi/vue';
import { LibLocalStorage } from '@zeedhi/tek-lib';
import { LoadingService } from '@zeedhi/common';
import Store from './store';

Vue.use(Router);

const router = new Router({
  mode: 'history',
  base: process.env.BASE_URL,
  routes: [
    {
      path: '/',
      name: 'home',
      component: ZdPage,
      props: () => ({
        path: 'home',
        local: true,
      }),
    },
    {
      path: '/modules/:module/:page',
      name: 'microservice',
      component: () => import('./views/Microservice.vue'),
    },
    {
      path: '*',
      component: ZdPage,
      props: {
        path: 'notfound',
        local: true,
      },
    },
    {
      path: '/ger21000_formas_apresentacao',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/ger21000_formas_apresentacao',
        local: true,
      }),
    },
    {
      path: '/pme00600_elaboracao_cardapio',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/cardapio/pme00600_elaboracao_cardapio',
        local: true,
      }),
    },
    {
      path: '/ger96000_central',
      name: 'Central',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/cadastro/ger96000_central',
        local: true,
      }),
    },
    {
      path: '/pla01300_prato',
      name: 'prato',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/pla01300_prato',
        local: true,
      }),
    },
    {
      path: '/pla01308_utilizacao_produto_receita',
      name: 'utilizacaoProdutos',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/pla01308_utilizacao_produto_receita',
        local: true,
      }),
    },
    {
      path: '/pla01303_emissao_das_receitas',
      name: 'emissaoReceitas',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/pla01303_emissao_das_receitas',
        local: true,
      }),
    },
    {
      path: '/pme11000_percentual_adesao_prato',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/parametrizacao/pme11000_percentual_adesao_prato',
        local: true,
      }),
    },
    {
      path: '/pme00900_requisicao_servico_analitica',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/requisicao/pme00900_requisicao_servico_analitica',
        local: true,
      }),
    },
    {
      path: '/pme00800_calculo_previsao_consumo',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/requisicao/pme00800_calculo_previsao_consumo',
        local: true,
      }),
    },
    {
      path: '/pme03100_emissao_card_dia',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/relatorios/pme03100_emissao_card_dia',
        local: true,
      }),
    },
    {
      path: '/pme03000_emissao_card_periodo',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/relatorios/pme03000_emissao_card_periodo',
        local: true,
      }),
    },
    {
      path: '/ger80500_nutricionista',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/ger80500_nutricionista',
        local: true,
      }),
    },
    {
      path: '/pme00300_tipo_de_cardapio',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/parametrizacao/pme00300_tipo_de_cardapio',
        local: true,
      }),
    },
    {
      path: '/pme00700_consumo_por_servico',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/servico/pme00700_consumo_por_servico',
        local: true,
      }),
    },
    {
      path: '/pme00200_servico',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/servico/pme00200_servico',
        local: true,
      }),
    },
    {
      path: '/report',
      name: 'report',
      component: ZdPage,
      props: () => ({
        path: 'report',
        local: true,
      }),
    },
    {
      path: '/pme01000_emissao_card_semana',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/relatorios/pme01000_emissao_card_semana',
        local: true,
      }),
    },
    {
      path: '/pme02400_clientes_por_servico',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/servico/pme02400_clientes_por_servico',
        local: true,
      }),
    },
    {
      path: '/pme09100_nutriente_cardapio',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/relatorios/pme09100_nutriente_cardapio',
        local: true,
      }),
    },
    {
      path: '/pme01000_emissao_card_semana',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/relatorios/pme01000_emissao_card_semana',
        local: true,
      }),
    },
    {
      path: '/pme02400_clientes_por_servico',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/servico/pme02400_clientes_por_servico',
        local: true,
      }),
    },
    {
      path: '/pme09100_nutriente_cardapio',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/relatorios/pme09100_nutriente_cardapio',
        local: true,
      }),
    },
    {
      path: '/pme09100_nutriente_cardapio',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/relatorios/pme09100_nutriente_cardapio',
        local: true,
      }),
    },
    {
      path: '/pme00500_comensais_estimados',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/parametrizacao/pme00500_comensais_estimados',
        local: true,
      }),
    },
    {
      path: '/pme08100_custo_real_periodo',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme08100_custo_real_periodo',
        local: true,
      }),
    },
    {
      path: '/pme01300_emissao_card_periodo_PAE',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/relatorios/pme01300_emissao_card_periodo_PAE',
        local: true,
      }),
    },
    {
      path: '/pme08000_custo_planejado_cardapio',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme08000_custo_planejado_cardapio',
        local: true,
      }),
    },
    {
      path: '/pme01400_efetivos',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/medicao/pme01400_efetivos',
        local: true,
      }),
    },
    {
      path: '/ger02500_nutrientes',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/ger02500_nutrientes',
        local: true,
      }),
    },
    {
      path: '/ger30000_padrao_de_servico',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/servico/ger30000_padrao_de_servico',
        local: true,
      }),
    },
    {
      path: '/pme02000_retirada_inventario',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/medicao/pme02000_retirada_inventario',
        local: true,
      }),
    },
    {
      path: '/pme01600_desvio_de_consumo',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme01600_desvio_de_consumo',
        local: true,
      }),
    },
    {
      path: '/pme01700_custo_realizado_global_do_periodo',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme01700_custo_realizado_global_do_periodo',
        local: true,
      }),
    },
    {
      path: '/pme01100_custo_planejado_realizado',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme01100_custo_planejado_realizado',
        local: true,
      }),
    },
    {
      path: '/pme01800_consumo_planejado_realizado',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme01800_consumo_planejado_realizado',
        local: true,
      }),
    },
    {
      path: '/pme08200_custo_card_mes',
      name: 'zeedhi',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme08200_custo_card_mes',
        local: true,
      }),
    },
  ],
});

const isOpeningNewModule = function isOpeningNewModule(to: any, from: any) {
  const isFromProduct = (to.name === 'microservice' && from.name !== 'microservice');
  // eslint-disable-next-line max-len
  const isFromAnotherModule = ((to.name === 'microservice' && from.name === 'microservice') && to.path.split('/')[2] !== from.path.split('/')[2]);
  const isNotLoginRoute = (to.fullPath !== Config.env.modules.login.routePath);
  return ((isFromProduct || isFromAnotherModule) && isNotLoginRoute);
};

// eslint-disable-next-line max-len
const callLibStartSessionFromModule = async function callLibStartSessionFromModule(to: any) {
  const data = {
    USER: localStorage.getItem('LOGIN_USER'),
    LOGIN_LANGUAGE: localStorage.getItem('LOGIN_LANGUAGE'),
    LOGIN_TOKEN: localStorage.getItem('LOGIN_TOKEN'),
  };
  await Http.post(`${Config.env.modules[to.path.split('/')[2]].endPoint}/lib_startSession`, data);
};

router.beforeEach(async (to, from, next) => {
  if (isOpeningNewModule(to, from)) {
    try {
      LoadingService.show();
      await callLibStartSessionFromModule(to);
      LoadingService.hide();
    } catch (e) {
      LoadingService.hide();
    }
  }

  LibLocalStorage.setItem('IS_NEXT', true);

  if (to.path !== Config.env.loginRoutePath && !localStorage.getItem('LOGIN_TOKEN')) {
    next(Config.env.loginRoutePath);
  } else if (to.path === Config.env.loginRoutePath && localStorage.getItem('LOGIN_TOKEN')) {
    next(Config.env.baseUrl);
  } else {
    next();
  }
});

router.afterEach((to, from) => {
  const htmlElement = document.getElementsByTagName('html')[0];
  if (to.name === 'microservice') {
    htmlElement.setAttribute('microservice', '');
    Store.commit('setRouterParams', to.params);
    const frame = (document.getElementById('microservice-frame') as HTMLIFrameElement);
    if (frame && frame.contentWindow && to.path.split('/')[2] === from.path.split('/')[2]) {
      frame.contentWindow.postMessage({
        type: 'microservice-route-change',
        routerParams: to.params,
      }, `${Config.env.modules[to.path.split('/')[2]].baseUrl}`);
    }
    window.document.title = `${Config.env.product.name} - ${to.params.page}`;
  } else {
    htmlElement.removeAttribute('microservice');
    window.document.title = Config.env.product.name;
  }

  if (to.name === 'login') {
    htmlElement.setAttribute('login', '');
  } else {
    htmlElement.removeAttribute('login');
  }
});

export default router;
